//------------Copyright (C) 2011 Maxim Integrated Products --------------
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL MAXIM INTEGRATED PRODCUTS BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Maxim Integrated Products
// shall not be used except as stated in the Maxim Integrated Products
// Branding Policy.
// ---------------------------------------------------------------------------

#ifndef __DS1WM_H
#define __DS1WM_H

#include "GenericTypeDefs.h"

#define MaxNumberDevices 20		//Maximum number of devices on 1-Wire (this is user defined).

//PROTOTYPES
void CommandRegisterWrite(BYTE);
BYTE CommandRegisterRead(void);
void TransmitBufferWrite(BYTE);
BYTE ReceiveBufferRead(void);
BYTE InterruptRegisterRead(void);
void InterruptEnableRegisterWrite(BYTE);
BYTE InterruptEnableRegisterRead(void);
void ClockDivisorRegisterWrite(BYTE);
BYTE ClockDivisorRegisterRead(void);
void ControlRegisterWrite(BYTE);
BYTE ControlRegisterRead(void);
void WriteByte2Register(BYTE,BYTE);
BYTE ReadByteFromRegister(BYTE);
int OWReset(void);
BYTE OWWriteByte(BYTE);
BYTE OWWriteByteSupply(BYTE);
BYTE OWWriteBytes(BYTE *, int);
BYTE OWReadByte(void);
void OWReadBytes(BYTE *,int);
int MatchROM(BYTE [][8],BYTE);
int OWSearch(BYTE [][8]);
int ROMCRCCheck(BYTE [][8],BYTE);
void SetClockFrequency(BYTE);
BYTE GetOneWireState(void);
BYTE PullOneWireLow(void);
BYTE ReleaseOneWire(void);
BYTE LongLineModeEnable(void);
BYTE LongLineModeDisable(void);
BYTE PresencePulseMaskingEnable(void);
BYTE PresencePulseMaskingDisable(void);
BYTE OverdriveEnable(void);
BYTE OverdriveDisable(void);
BYTE StrongPullupEnable(void);
BYTE StrongPullupDisable(void);
BYTE StrongPullupSupplyEnable(void);
BYTE StrongPullupSupplyDisable(void);
BYTE BitControlEnable(void);
BYTE BitControlDisable(void);

void InitializePorts(void);
void FlushRegisters(void);

//** REGISTER DEFINITIONS *******************************************************
//Register Addresses
#define CommandRegister				0x00
#define TransmitBufferRegister		0x01
#define InterruptRegister			0x02
#define InterruptEnableRegister		0x03
#define ClockDivisorRegister		0x04
#define ControlRegister				0x05

//Interrupt Register Bit Locations
#define	PD			0x01	//Presence Detect
#define	PDR			0x02	//Presence Detect Result
#define	TBE			0x04	//Transmit Buffer Empty
#define	TEMT		0x08	//Transmit Shift Register Empty
#define	RBF			0x10	//Receive Buffer Full
#define	RSRF		0x20	//Receive Shift Register Full
#define	OW_SHORT	0x40	//One Wire Short
#define	OW_LOW		0x80	//One Wire Low

//Command Register Bit Locations
//NOTE: Only four bits are used in this register.
#define	OneWR		0x01	//1-Wire Reset (1WR)
#define	SRA			0x02	//Search ROM Accelerator
#define	FOW			0x04	//Force One Wire
#define	OWIN		0x08	//One Wire Input

//Interrupt Enable Register
#define EPD			0x01	//Enable Presence Detect Interrupt
#define IAS			0x02	//INTR Active State
#define ETBE		0x04	//Enable Transmit Buffer Empty Interrupt
#define ETMT		0x08	//Enable Transmit Shift Register Empty
#define ERBF		0x10	//Enable Receive Buffer Full Interrupt
#define ERSF		0x20	//Enable Receive Shift Register Full Interrupt
#define EOWSH		0x40	//Enable One Wire Short Interrupt
#define EOWL		0x80	//Enable One Wire Low Interrupt

//Control Register Bit Locations
//NOTE: The MSbit in this register is not used.
#define LLM			0x01	//Long Line Mode
#define PPM			0x02	//Presence Pulse Masking
#define ENFOW		0x04	//Enable Force One Wire
#define STPEN		0x08	//Strong Pullup Enable
#define STPSPLY		0x10	//Strong Pullup Supply
#define BITCTL		0x20	//Bit Control
#define OD			0x40	//Overdrive

//** PIN DEFINITIONS *******************************************************
//Data Bus: This bus comprises eight input/output lines. The bus provides bi-directional
//communications between the 1-Wire master and the CPU. Data, control words, and status
//information are transferred via this D7-D0 Data Bus.
//NOTE: If this port is changed, the TRISx statments in the functions need to change too.
//(INPUT/OUTPUT)
#define DataBus		PORTD	//8-Bit Bi-Directional Bus

//Register Select: Address signals connected to these three inputs select a register
//for the CPU to read from or write to during data transfer. A table of registers and
//their addresses is shown below. (OUTPUT)
//Register Addresses
//A2 A1 A0 Register
//0  0  0  Command Register (read/write)
//0  0  1  Transmit Buffer (write), Receive Buffer (read)
//0  1  0  Interrupt Register (read)
//0  1  1  Interrupt Enable Register (read/write)
//1  0  0  Clock Divisor Register (read/write)
//1  0  1  Control Register (read/write)
#define A0			PORTBbits.RB2
#define A1			PORTBbits.RB3
#define A2			PORTBbits.RB6

//Address Strobe: The positive edge of an active Address Strobe (ADS) signal latches
//the Register Select (A0, A1, A2) into an internal latch. Provided that setup and
//hold timings are observed, ADS may be tied low making the latch transparent.
//Pin Type: Output
#define ADS_BAR		PORTEbits.RE2

//Enable: When EN is low, the 1-Wire master is enabled; this signal acts as the device
//chip enable. This enables communication between the 1-Wire master and the CPU.
//Pin Type: Output
#define EN_BAR		PORTCbits.RC2


//Interrupt: This line goes to its active state whenever any one of the interrupt types
//has an active high condition and is enabled via the Interrupt Enable Register. The
//INTR signal is reset to an inactive state when the Interrupt Register is read.
//NOTE: Map INTR to a hardware interrupt pin if polling will not be used.
//Pin Type: Input
#define INTR		PORTCbits.RC2

//Master Reset: When this input is high, it clears all the registers and the control
//logic of the 1-Wire master, and sets INTR to its default inactive state, which is HIGH.
//Pin Type: Output
#define MR			PORTBbits.RB0
// BANK2_SW

//Write: This pin drives the bus during a write cycle. When WR is low while the circuit
//is enabled, the CPU can write control words or data into the selected register. RD
//and WR should never be low simultaneously; if they are, WR takes precedence.
//Pin Type: Output
#define WR_BAR		PORTAbits.RA6

//Read: This pin drives the bus during a read cycle. When the circuit is enabled, the
//CPU can read status information or data from the selected register by driving RD
//low. RD and WR should never be low simultaneously; if they are, WR takes precedence.
//Pin Type: Output
#define RD_BAR		PORTCbits.RC0

#endif